class AuthorsController < ApplicationController
  layout 'test' if MagicNumber['test_layout']
  before_filter :authenticate_user!, :only => [:index, :show, :new, :create, :edit, :update]
  before_filter :authenticate_author, :only => [:index, :show, :edit, :update]
  before_filter :authenticate_admin!, :only => [:list, :browse]

  def index
    @page = Author.page params[:page]
    @page_size = Author.page_size params[:page_size]
    @authors = Author.list(@page, @page_size)

    respond_to do |format|
      format.html # index.html.erb
      format.json { render :json => @authors.to_json(Author.list_json_opt) }
    end
  end

  def show
    @au = Author.show(params[:id], @author)

    respond_to do |format|
      format.html
      format.json { render :json => @au.to_json(Author.show_json_opt) }
    end
  end

  def count
    @au = {:count => Author.visible_count}
    respond_to do |format|
      format.json { render json: @au.to_json }
    end
  end
  
  def list
    @authors = Author.all

    respond_to do |format|
      format.html { render layout: 'system' }
    end
  end

  def browse
    @au = Author.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' }
    end
  end
  
  def new
    @au = Author.new
    @au.supply_default 

    respond_to do |format|
      format.html # new.html.erb
      format.js
      format.json { render json: @au.to_json(Author.show_json_opt) }
    end
  end

  def edit
    @au = Author.edit(params[:id], @author)
  end

  def create
    @au = Author.new()
    @au.supply_default 
    @au.attributes = params[:author]
    @au.overwrite @user.id
    respond_to do |format|
      if @au.save
        flash[:notice] = I18n.t('flash.notice.created', :model => Author.model_name.human)
        format.html { redirect_to root_path }
        format.json { render json: @au.to_json(Author.show_json_opt), status: :created, location: @artist }
      else
        format.html { render action: "new" }
        format.json { render json: @au.errors, status: :unprocessable_entity }
      end
    end
  end

  def update
    @au = Author.edit(params[:id], @author)
    @au.attributes = params[:author]
    @au.overwrite @user.id

    respond_to do |format|
      if @au.save
        flash[:notice] = I18n.t('flash.notice.updated', :model => Author.model_name.human)
        format.html { redirect_to '/home/configure' }
        format.json { head :ok }
      else
        format.html { render action: "edit" }
        format.json { render json: @au.errors, status: :unprocessable_entity }
      end
    end
  end
end
