class ResourcePicturesController < ApplicationController
  layout 'test' if MagicNumber['test_layout']
  before_filter :authenticate_user!, :only => [:index, :show, :credit, :new, :create, :update, :destroy]
  before_filter :authenticate_author, :only => [:index, :show, :credit, :new, :create, :update, :destroy]
  before_filter :authenticate_admin!, :only => [:list, :browse]
  before_filter :authenticate_artist, :only => [:new, :create, :destroy]

  # GET /resource_pictures
  # GET /resource_pictures.json
  def index
    @page = ResourcePicture.page params[:page]
    @page_size = ResourcePicture.page_size params[:page_size]
    @resource_pictures = ResourcePicture.list(@page, @page_size)

    respond_to do |format|
      format.html # index.html.erb
      format.json { render :json => @resource_pictures.to_json(ResourcePicture.list_json_opt) }
    end
  end

  # GET /resource_pictures/1
  # GET /resource_pictures/1.json
  def show
    @resource_picture = ResourcePicture.show(params[:id], @author)

    respond_to do |format|
      opt = {:type => @resource_picture.mime_type, :disposition=>"inline"}
      format.png { send_data(@resource_picture.restore(params[:subdir]), opt ) }
      format.gif { send_data(@resource_picture.restore(params[:subdir]), opt ) }
      format.jpeg { send_data(@resource_picture.restore(params[:subdir]), opt ) }
      format.html # show.html.erb
      format.json { render :json => @resource_picture.to_json(ResourcePicture.show_json_opt)}
    end
  end
  
  def credit
    @resource_picture = ResourcePicture.show(params[:id], @author)

    respond_to do |format|
      format.html { render :layout => false } # show.html.erb
      format.json { render :json => @resource_picture.to_json(ResourcePicture.show_json_opt)}
    end
  end
  
  def new
    @original_picture = OriginalPicture.edit params[:original_picture_id], @artist
    @imager = PettanImager.load @original_picture.restore
    @original_picture_license_group = OriginalPictureLicenseGroup.new params[:original_picture_license_group]
    @license_group = LicenseGroup.show @original_picture_license_group.license_group_id
    @resource_picture = @original_picture.resource_picture
    unless @resource_picture
      @resource_picture = ResourcePicture.new
      @resource_picture.supply_default
    end
    @resource_picture.attributes = params[:resource_picture]
    @resource_picture.overwrite @original_picture
    @license = @resource_picture.license
    
    respond_to do |format|
      format.html # new.html.erb
      format.js
    end
  end
  
  def create
    @original_picture = OriginalPicture.edit params[:original_picture_id], @artist
    @imager = PettanImager.load @original_picture.restore
    @original_picture_license_group = OriginalPictureLicenseGroup.new params[:original_picture_license_group]
    @license_group = LicenseGroup.show @original_picture_license_group.license_group_id
    @resource_picture = @original_picture.resource_picture
    unless @resource_picture
      @resource_picture = ResourcePicture.new
      @resource_picture.supply_default
    end
    @resource_picture.attributes = params[:resource_picture]
    @resource_picture.overwrite @original_picture
    @license = @resource_picture.license

    respond_to do |format|
      if @resource_picture.store(@imager)
        format.html { redirect_to @resource_picture }
        format.json { render json: @resource_picture.to_json(ResourcePicture.show_json_opt), status: :created, location: @resource_picture }
      else
        format.html { render action: "new" }
        format.json { render json: @resource_picture.errors, status: :unprocessable_entity }
      end
    end
  end
  
  def destroy
    @resource_picture = ResourcePicture.edit(params[:id], @artist)

    respond_to do |format|
      if @resource_picture.unpublish
        format.html { redirect_to :controller => '/home', :action => :resource_picture }
        format.json { head :ok }
      else
        format.html { redirect_to resource_picture_path(@resource_picture) }
        format.json { render json: @resource_picture.errors, status: :unprocessable_entity }
      end
    end
  end
  
  def count
    @resource_picture = {:count => ResourcePicture.visible_count}
    respond_to do |format|
      format.json { render json: @resource_picture.to_json }
    end
  end
  
  def list
    @resource_pictures = ResourcePicture.all

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @resource_pictures }
    end
  end

  def browse
    @resource_picture = ResourcePicture.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' }
      format.json { render json: @resource_picture }
    end
  end

end
