class Demander < ActiveRecord::Base
  belongs_to :demand_user
  belongs_to :demander_status
  
  validates :demand_user_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :name, :presence => true, :length => {:maximum => 50}, :uniqueness => true
  validates :caption, :presence => true, :length => {:maximum => 30}
  validates :url, :presence => true, :length => {:maximum => 200}, :url => {:message => I18n.t('errors.messages.url')}
  validates :description, :presence => true
  validates :demander_url, :presence => true, :length => {:maximum => 200}, :url => {:message => I18n.t('errors.messages.url')}
  validates :demander_status_id, :presence => true, :numericality => true, :existence => {:both => false}
  
  def supply_default
  end
  
  def overwrite du
    self.demand_user_id = du.id
  end
  
  def own? du
    return false unless du.is_a?(DemandUser)
    du.id == self.demand_user_id
  end
  
  def visible? du
    return false unless du.is_a?(DemandUser)
    du.id == self.demand_user_id
  end
  
  def status
    self.demander_status.status
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.default_panel_size
    30
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(Demander.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:order => 'demanders.name'})
    Demander.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {:demander_status => {}} }
  end
  
  def self.list_json_opt
    {:include => {:demander_status => {}} }
  end
  
  def self.show pid, du
    opt = {}
    opt.merge!(Demander.show_opt)
    res = Demander.find(pid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(du)
    res
  end
  
  def self.edit pid, du
    opt = {}
    opt.merge!(Demander.show_opt)
    res = Demander.find(pid, opt)
    raise ActiveRecord::Forbidden unless res.own?(du)
    res
  end
  
  def self.show_opt
    {:include => {:demander_status => {}} }
  end
  
  def self.show_json_opt
    {:include => {:demander_status => {}} }
  end
  
  def store
    r = false
    Demander.transaction do
      if self.new_record?
        ds = DemanderStatus.new
        ds.supply_default
        if ds.save
          self.demander_status_id = ds.id
        else
          self.demander_status_id = nil
       end
      end
      r = self.save
      raise ActiveRecord::Rollback unless r
    end
    r
  end
  
  def stop
    self.demander_status.stop
  end
  
end
